c    "Copyright 2007 Prof. Robert Dalrymple, Prof. M. Gomez Gesteira, Dr Benedict Rogers, 
c     Alejandro Crespo, Muthukumar Narayanaswamy, Dr Shan Zou, Dr Andrea Panizzo "
c
c    This file is part of SPHYSICS.
c
c    SPHYSICS is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 3 of the License, or
c    (at your option) any later version.
c
c    SPHYSICS is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program.  If not, see <http://www.gnu.org/licenses/>.

      subroutine updateNormals_3D(i_start, i_stop)

      include 'common.3D'
      
c       Subroutine calculates tangents and normals for a surface given 
c       by three points:
c
c       Vectors t, s, n form a righ-handed (orthogonal) system
c
c
c       (xi     , yi    , zi )    = point of interest 
c       (xt_next,yt_next,zt_next) = next point in traverse of plane
c       (xs_next,ys_next,zs_next) = corresponding point in orthogonal traverse

!               ^ (drs)
!              /
!            
!       ^(n) o(3)
!       |   /
!       |  /
!       | /
!       |/ )
!    (i)o---------o(2)  -->(drt)
c    
c      i_start = 1         !BedPs only
c      i_stop  = nbf       !Bed particles only
c
c      i_start = nbf + 1   !Moving Solid Object particles only
c      i_stop  = nb        !Moving Solid Object particles only
c
c       Specifically written for use in Monaghan's BC

    
      !-- Serial Version --
      do i = i_start,i_stop  
        xp_i = xp(i)
        yp_i = yp(i)
        zp_i = zp(i)
        
        iminus1_neighbour = iBP_Pointer_Info(i,3)
        iplus1_neighbour  = iBP_Pointer_Info(i,4)
        jminus1_neighbour = iBP_Pointer_Info(i,5)
        jplus1_neighbour  = iBP_Pointer_Info(i,6)

        
        xp_BP_iminus1 = xp(iminus1_neighbour)
        yp_BP_iminus1 = yp(iminus1_neighbour)
        zp_BP_iminus1 = zp(iminus1_neighbour)
        xp_BP_iplus1 = xp(iplus1_neighbour)
        yp_BP_iplus1 = yp(iplus1_neighbour)
        zp_BP_iplus1 = zp(iplus1_neighbour)

        xp_BP_jminus1 = xp(jminus1_neighbour)
        yp_BP_jminus1 = yp(jminus1_neighbour)
        zp_BP_jminus1 = zp(jminus1_neighbour)
        xp_BP_jplus1 = xp(jplus1_neighbour)
        yp_BP_jplus1 = yp(jplus1_neighbour)
        zp_BP_jplus1 = zp(jplus1_neighbour)
        
        if(i.eq.iminus1_neighbour)then
          xt_prev = xp_i - (xp_BP_iplus1 - xp_i)
          yt_prev = yp_i - (yp_BP_iplus1 - yp_i)
          zt_prev = zp_i - (zp_BP_iplus1 - zp_i)
          xt_next = xp_BP_iplus1
          yt_next = yp_BP_iplus1
          zt_next = zp_BP_iplus1
        else if(i.eq.iplus1_neighbour)then
          xt_prev = xp_BP_iminus1
          yt_prev = yp_BP_iminus1
          zt_prev = zp_BP_iminus1
          xt_next = xp_i + (xp_i - xp_BP_iminus1)
          yt_next = yp_i + (yp_i - yp_BP_iminus1)
          zt_next = zp_i + (zp_i - zp_BP_iminus1)
        else
          xt_prev = xp_BP_iminus1
          yt_prev = yp_BP_iminus1
          zt_prev = zp_BP_iminus1
          xt_next = xp_BP_iplus1
          yt_next = yp_BP_iplus1
          zt_next = zp_BP_iplus1
        end if
        if(i.eq.jminus1_neighbour)then
          xs_prev = xp_i - (xp_BP_jplus1 - xp_i)
          ys_prev = yp_i - (yp_BP_jplus1 - yp_i)
          zs_prev = zp_i - (zp_BP_jplus1 - zp_i)
          xs_next = xp_BP_jplus1
          ys_next = yp_BP_jplus1
          zs_next = zp_BP_jplus1
        else if(i.eq.jplus1_neighbour)then
          xs_prev = xp_BP_jminus1
          ys_prev = yp_BP_jminus1
          zs_prev = zp_BP_jminus1
          xs_next = xp_i + (xp_i - xp_BP_jminus1)
          ys_next = yp_i + (yp_i - yp_BP_jminus1)
          zs_next = zp_i + (zp_i - zp_BP_jminus1)
        else
          xs_prev = xp_BP_jminus1
          ys_prev = yp_BP_jminus1
          zs_prev = zp_BP_jminus1
          xs_next = xp_BP_jplus1
          ys_next = yp_BP_jplus1
          zs_next = zp_BP_jplus1
        end if
        
c        xtb,ytb,ztb are components of tangential
c       
        dxt_fwd = xt_next - xp_i
        dyt_fwd = yt_next - yp_i
        dzt_fwd = zt_next - zp_i
        dxt_back = (xp_i - xt_prev)
        dyt_back = (yp_i - yt_prev)
        dzt_back = (zp_i - zt_prev)

c        call periodicityCheck3D(dxt_fwd,  dyt_fwd , dyt_fwd ,
c     &                 xp_i, xt_next, yp_i, yt_next, zp_i, zt_next)
c        call periodicityCheck3D(dxt_back, dyt_back, dyt_back,
c     &                 xp_i, xt_prev, yp_i, yt_prev, zp_i, zt_prev)
        dxt = dxt_fwd + dxt_back
        dyt = dyt_fwd + dyt_back
        dzt = dzt_fwd + dzt_back

c        xsb,ysb,zsb are components of orthogonal tangential

c              
        dxs_fwd = xs_next - xp_i
        dys_fwd = ys_next - yp_i
        dzs_fwd = zs_next - zp_i
        dxs_back = (xp_i - xs_prev)
        dys_back = (yp_i - ys_prev)
        dzs_back = (zp_i - zs_prev)
        
c        call periodicityCheck3D(dxs_fwd,  dys_fwd , dys_fwd ,
c     &                 xp_i, xs_next, yp_i, ys_next, zp_i, zs_next)
c        call periodicityCheck3D(dxs_back, dys_back, dys_back,
c     &                 xp_i, xs_prev, yp_i, ys_prev, zp_i, zs_prev)
        dxs = dxs_fwd + dxs_back
        dys = dys_fwd + dys_back
        dzs = dzs_fwd + dzs_back

c       normal = vec(drt) x vec(drs)     :vector cross-product

        dxt_c = dxt_fwd + dxt_back
        dyt_c = dyt_fwd + dyt_back
        dzt_c = dzt_fwd + dzt_back
        dxs_c = dxs_fwd + dxs_back
        dys_c = dys_fwd + dys_back
        dzs_c = dzs_fwd + dzs_back

        dxn = dyt_c*dzs_c - dzt_c*dys_c   !dyt*dzr - dzt*dyr
        dyn = dzt_c*dxs_c - dxt_c*dzs_c   !dzt*dxr - dxt*dzr
        dzn = dxt_c*dys_c - dyt_c*dxs_c   !dxt*dyr - dyt*dxr
        
        drt = sqrt(dxt*dxt + dyt*dyt + dzt*dzt)
        xtb(i) = dxt/drt
        ytb(i) = dyt/drt
        ztb(i) = dzt/drt
        
        drs = sqrt(dxs*dxs + dys*dys + dzs*dzs)
        xsb(i) = dxs/drs
        ysb(i) = dys/drs
        zsb(i) = dzs/drs

        
c       normal = vec(drt) x vec(drs)     :vector cross-product
        drn = sqrt(dxn*dxn + dyn*dyn + dzn*dzn)
        xnb(i) = dxn/drn
        ynb(i) = dyn/drn
        znb(i) = dzn/drn

        deltaptb(i,1) = sqrt((dxt_back)**2 + (dyt_back)**2 
     &                     + (dzt_back)**2)
        deltaptb(i,2) = sqrt((dxt_fwd)**2 + (dyt_fwd)**2
     &                     + (dzt_fwd)**2)
        deltapsb(i,1) = sqrt((dxs_back)**2 + (dys_back)**2 
     &                     + (dzs_back)**2)
        deltapsb(i,2) = sqrt((dxs_fwd)**2 + (dys_fwd)**2
     &                     + (dzs_fwd)**2)

      
      enddo


      end subroutine
